"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.format_stream_list = exports.error_entry = exports.add_rows_padding = exports.format_header = exports.format_title = exports.format_init_header = exports.read_styles = exports.read_config = exports.read_json = void 0;
const CONF_PATH = __dirname.substr(0, __dirname.length - 6) + 'conf/';
function read_json(path) {
    try {
        const file = require('fs').readFileSync(path);
        return JSON.parse(file);
    }
    catch (err) {
        throw Error(`Couldn't load ${path}: ${err})`);
    }
}
exports.read_json = read_json;
function read_config() {
    return read_json(CONF_PATH + 'config.json');
}
exports.read_config = read_config;
function read_styles() {
    return read_json(CONF_PATH + 'styles.json');
}
exports.read_styles = read_styles;
function format_init_header(option) {
    const def_style = '{white-bg}{black-fg}';
    const pad = ' ';
    const line = Object.keys(option).reduce((acc, key) => acc + ` ${def_style} ${key} {/} ${option[key]}${pad}`, '');
    return line.substr(0, line.length - 1);
}
exports.format_init_header = format_init_header;
function format_title(s, name) {
    return `Net streams - ${s.flags.source}` + (name === '' ? '' : ` | ${name}`);
}
exports.format_title = format_title;
function format_header(tab, option, pause_key, is_paused) {
    const def_style = '{white-bg}{black-fg}';
    const sel_style = '{green-bg}{black-fg}';
    const pad = ' ';
    const line = Object.keys(option).reduce((acc, key) => {
        const letter = key;
        const text = letter === pause_key
            ? !is_paused
                ? 'Pause'
                : 'Resume'
            : option[key];
        const style = text === tab ? sel_style : def_style;
        return acc + ` ${style} ${letter} {/} ${text}${pad}`;
    }, '');
    return line.substr(0, line.length - 1);
}
exports.format_header = format_header;
function add_rows_padding(rows_header, rows) {
    const pad = '  ';
    return [rows_header].concat(rows).map((arr) => arr.map((cell) => pad + cell));
}
exports.add_rows_padding = add_rows_padding;
function error_entry(error, source = '') {
    return {
        name: error,
        homepage: '',
        listeners: '',
        description: '',
        playing: '',
        url: '',
        bitrate: '',
        src: source,
        is_playlist: false,
        entry: ''
    };
}
exports.error_entry = error_entry;
function format_bitrate(bitrate) {
    const bitrate_pad = '   ';
    const pad = bitrate_pad.substr(bitrate.length);
    return !bitrate || bitrate === '0' ? '' : `${pad}${bitrate} kbps`;
}
function format_icecast_list(rows_header, list) {
    const char_limit = {
        name: 30,
        playing: 50,
        description: 50,
        listeners: 20,
        bitrate: 20
    };
    return add_rows_padding(rows_header, list.map(entry => [
        entry.name.substr(0, char_limit.name),
        entry.playing.substr(0, char_limit.playing),
        entry.listeners.substr(0, char_limit.listeners)
    ]));
}
function format_shoutcast_list(rows_header, list) {
    const char_limit = {
        name: 30,
        playing: 50,
        description: 50,
        listeners: 20,
        bitrate: 20
    };
    return add_rows_padding(rows_header, list.map(entry => [
        entry.name.substr(0, char_limit.name),
        entry.playing.substr(0, char_limit.playing),
        entry.listeners.substr(0, char_limit.listeners),
        format_bitrate(entry.bitrate).substr(0, char_limit.bitrate)
    ]));
}
function format_radio_list(rows_header, list) {
    const char_limit = {
        name: 50,
        bitrate: 20
    };
    return add_rows_padding(rows_header, list.map(entry => [
        entry.name.substr(0, char_limit.name),
        format_bitrate(entry.bitrate).substr(0, char_limit.bitrate)
    ]));
}
function format_stream_list(s, list, rows_header, search) {
    switch (list.length > 0) {
        case true:
            const src = list[0].src;
            switch (src) {
                case 'Icecast':
                    return format_icecast_list(rows_header, list);
                case 'Shoutcast':
                    return format_shoutcast_list(rows_header, list);
                case 'Radio':
                    return format_radio_list(rows_header, list);
                default:
                    throw Error(`Unknown source: ${src} ${s}`);
            }
        default:
            return add_rows_padding(rows_header, [['No results for: ' + search]]);
    }
}
exports.format_stream_list = format_stream_list;
